#ifndef CODING_DRONE_SW_H_
#define CODING_DRONE_SW_H_

#include <avr/io.h>
#include <avr/pgmspace.h>

#define  VERSION        001


class CodingDrone
{
public:
  CodingDrone();
  
  typedef struct {
	uint8_t Up;
	uint8_t Down;
	uint8_t Left;
	uint8_t Right;
	uint8_t Forward;
	uint8_t Backward;
	uint8_t CW_Rotation;
	uint8_t CCW_Rotation;
	uint8_t None;
  } Control_Direction;

  typedef struct {
	uint8_t TakeOff;
	uint8_t Landing;
	uint8_t Emergency;
	uint8_t None;
  } Control_System;
  
  Control_Direction Direction;
  Control_System System;
  
  void begin();  
  void MotorSetting(int M_1, int M_2, int M_3, int M_4);
  void LED_OFF(int Pin);
  void LED_ON(int Pin);  
  
  void setTrim(int x, int y);
  float readBattery(uint16_t VB);
  void FailSafe();
  void FailSafe_NoneComm();
  void readRemoteData();
  void Auto_Flight_Enable(int gg);
  void sensorCalibration();
  //uint8_t readCmd(uint8_t address ,int data);
  
  
  void Bluetooth_Setting(String s);
  void readSensorData();
  int Sensor_Data(uint8_t addr);
  void Auto_PID_Control(int Roll_Data, int Pitch_Data, int Yaw_Data, int Alt_Data);
  void AutoControl_None_Comm();
  
  void Motor_LeftRight(int speed_1, int speed_2, int speed_3, int speed_4);
  void Motor_ForwardBackward(int speed_1, int speed_2, int speed_3, int speed_4);
  void Motor_Rotation(int speed_1, int speed_2, int speed_3, int speed_4);
  void Motor_UpDown(int speed_1, int speed_2, int speed_3, int speed_4);
  void Motor(int speed_1, int speed_2, int speed_3, int speed_4);
  
  void TakeOff(int height);
  void Landing();
  void EmergencyStop();  
  
  int angle_ROLL();
  int angle_PITCH();
  int angle_YAW();
  int ALTITUDE_Data();
  int sensorData;
 
  int cmd[8];
  int8_t Drone_State_Flag;
  bool sensorCalibrationSwitch;
  
  bool BT_Available();
  void BT_SerialWrite(uint8_t c); 
  void BT_SerialWriteString(String s); 
  uint8_t BT_SerialRead(); 
  
  //int Drone_Motor_1;
  //int Drone_Motor_2;
  //int Drone_Motor_3;
  //int Drone_Motor_4;
  int motor_Data[4][4];
  int motor_Data2[4];
  uint8_t motor_addr[4];
  uint8_t Motor_select(uint8_t c);
  void Drone_analogWrite(int pin, int pwm);
  
  uint8_t Drone_motor_Flag;
  //uint8_t Drone_motor_Check_Flag;
  
  uint8_t cmd_select(uint8_t c);
  uint8_t cmd_sel[10];
  
private:

  int16_t rcData[8]; // interval [1000;2000]
  int motor[8]; // Motor 0 : FRONT_L / Motor 1 : FRONT_R / Motor 2 : REAR_L / Motor 3 : REAR_R
  int read_CMD_Done; // 전송카운트
  int take_off_height; // 전송카운트
  
  enum Drone_Protocol_Sequence {
    IDLE,
    HEADER_SIZE,
    HEADER_CONTROL,
    HEADER_CMD
  };
  
  uint8_t MODE;   // MODE 0 : DRONE_MODE  /  MODE 1 : EDUCATION_MODE
  uint8_t rcSerialCount;   // a counter to select legacy RX when there is no more MSP rc serial data
  uint8_t rcUnCorrectCount;   // Continuous Error Count
  
    // *************************
	// IN / OUT PIN
	// *************************

  int Pin_Motor_FL; // FL : Front Left 전방-왼쪽
  int Pin_Motor_FR; // FR : Front Right 전방-오른쪽
  int Pin_Motor_RL; // RL : Rear Left 후방-왼쪽
  int Pin_Motor_RR; // RR : Rear Right 후방-오른쪽

  int Pin_VBAT; // 배터리 전압 체크

  int Pin_LED_R; // LED 1
  int Pin_LED_G; // LED 2
  int Pin_LED_B; // LED 3
  int Pin_LED_Y; // LED 4
  
  //void writeAllMotors(int16_t mc);
  
  int readPitchCommand;
  
  //BNC_CodingDrone.cpp
  void Initialize_BT();
  void Initialize_InOutPins();
  
  //Output.cpp
  //void MANUAL_MOTOR_CONTROL(uint8_t number, int Value);
  //void DRONE_MOTOR_CONTROL();
  
  //Process.cpp
  void go_arm();
  void go_disarm();
  void DRONE_SYSTEM_PERIOD_LOOPTIME();
  void DRONE_SYSTEM_PERIOD_20ms();
  void init_Drone_Setting();
  //void POSITION_CONTROL();
  
  void writeMoveCmd_Pitch(int data);
  void writeMoveCmd_Roll(int data);
  void writeMoveCmd_Yaw(int data);
  void writeMoveCmd_Throttle(int data);
  void writeMoveCmd_System(int data);
  
  //void BAT_Data_to_App_Bluetooth();
  void System_LED_Control();
  void ALTITUDE_PROCESS();
  
  //Sensor.cpp
  void initSensors();
  void Attitude_by_MPU6050();
  void Attitude_PID();
  void MPU6050_ACC_READ();
  void MPU6050_GYRO_READ();
  
  void getEstimatedAttitude();
    
};




class CodingDrone_AutoFlight
{
	CodingDrone DroneMainFunction;
public:
  CodingDrone_AutoFlight();
  void setTrim(int x, int y);
  void begin();  
  void Start();
  void End();  
  void TakeOff(int height);  
  void Landing();  
  void Hold(float sec);
  void Left(float sec);
  void Right(float sec);
  void Forward(float sec);
  void Backward(float sec);
  void CW_Rotation(float sec);
  void CCW_Rotation(float sec);
  void Up(float sec);
  void Down(float sec);
  
private:
  
  float datadata;

  bool Loop_Stop_Flag;
  uint32_t moveCurrentTime;
  uint16_t movePreviousTime;
  uint16_t moveCycleTime;     // this is the number in micro second to achieve a full loop, it can differ a little and is taken into account in the PID loop
  uint16_t moveCycleTime_TOTAL; 
  
  void BackgroundProcess();  

};


#endif /* CODING_DRONE_SW_H_ */


