/*************************************************************************
* File Name          : baseboard_firmware.ino
* Author             : Ander, Mark Yan
* Updated            : Ander, Mark Yan
* Version            : V0b.01.104
* Date               : 07/06/2016
* Description        : Firmware for Makeblock Electronic modules with Scratch.  
* License            : CC-BY-SA 3.0
* Copyright (C) 2013 - 2016 Maker Works Technology Co., Ltd. All right reserved.
* http://www.makeblock.cc/
**************************************************************************/

#include <Arduino.h>
#include "BNC_CodingDrone_Firmware.h"
#include "BNC_CodingDrone.h"

CodingDrone FirmwareDrone;

uint8_t drone_Sensor_flag = 0;
uint8_t drone_Command_flag = 0;

float Sensor_Data[4] = {0,};
int Control_Data[5] = {0,};

union{
    byte byteVal[4];
    float floatVal;
    long longVal;
}val;

union{
  byte byteVal[8];
  double doubleVal;
}valDouble;

union{
  byte byteVal[2];
  short shortVal;
}valShort;

#if defined(__AVR_ATmega32U4__) 
  int analogs[12]={A0,A1,A2,A3,A4,A5};
#endif
#if defined(__AVR_ATmega328P__) or defined(__AVR_ATmega168__)
  int analogs[8]={A0,A1,A2,A3,A4,A5,A6,A7};
#endif
#if defined(__AVR_ATmega1280__)|| defined(__AVR_ATmega2560__)
  int analogs[16]={A0,A1,A2,A3,A4,A5,A6,A7,A8,A9,A10,A11,A12,A13,A14,A15};
#endif
String mVersion = "0b.01.104";
boolean isAvailable = false;
boolean isBluetooth = false;

int len = 52;
char buffer[32];
char bufferBt[32];
byte index = 0;
byte dataLen;
byte modulesLen=0;
boolean isStart = false;
char serialRead;
#define VERSION 0
#define ULTRASONIC_SENSOR 1
#define TEMPERATURE_SENSOR 2
#define LIGHT_SENSOR 3
#define POTENTIONMETER 4
#define JOYSTICK 5
#define GYRO 6
#define SOUND_SENSOR 7
#define RGBLED 8
#define SEVSEG 9
#define MOTOR 10
#define SERVO 11
#define ENCODER 12
#define IR 13
#define IRREMOTE 14
#define PIRMOTION 15
#define INFRARED 16
#define LINEFOLLOWER 17
#define IRREMOTECODE 18
#define SHUTTER 20
#define LIMITSWITCH 21
#define BUTTON 22
#define HUMITURE 23
#define FLAMESENSOR 24
#define GASSENSOR 25
#define COMPASS 26
#define DIGITAL 30
#define ANALOG 31
#define PWM 32
#define SERVO_PIN 33
#define TONE 34
#define PULSEIN 37
#define ULTRASONIC_ARDUINO 36
#define STEPPER 40
#define LEDMATRIX 41
#define TIMER 50
#define TOUCH_SENSOR 51

#define DRONE_SENSOR_INITIALIZE 60
#define DRONE_SENSOR_RESET_SENSOR 61
#define DRONE_SENSOR_READ_ANGLE 62
#define DRONE_SENSOR_READ_ALT 63
#define DRONE_SENSOR_BT_AVAILABLE 64
#define DRONE_SENSOR_READ_BT 65
#define DRONE_SENSOR_SEND_BT 66
#define DRONE_SENSOR_BT_SETNAME 67
#define DRONE_SENSOR_BT_PARSING_DATA 68
#define DRONE_SENSOR_BT_CMD_READ 69
#define DRONE_SENSOR_READ_SENSOR_DATA 70
#define DRONE_SENSOR_READ_SENSOR 71
#define DRONE_MOTOR_CONTROL 72

#define GET 1
#define RUN 2
#define RESET 4
#define START 5
float angleServo = 90.0;
int servo_pins[8]={0,0,0,0,0,0,0,0};
unsigned char prevc=0;
double lastTimeTime = 0.0;
double currentTimeTime = 0.0;
uint8_t keyPressed = 0;
uint8_t command_index = 0;

unsigned char readBuffer(int index){
 return buffer[index]; 
}

void writeBuffer(int index,unsigned char c){
 if(isBluetooth){
  bufferBt[index]=c;
 }else{
  buffer[index]=c;
 } 
}

void writeHead(){
  writeSerial(0xff);
  writeSerial(0x55);
}

void writeEnd(){
  Serial.println(); 
}

void writeSerial(unsigned char c){
 Serial.write(c);
}

void readSerial(){
  isAvailable = false;
  if(Serial.available()>0){
    isAvailable = true;
    isBluetooth = false;
    serialRead = Serial.read();
  }
}
/*
ff 55 len idx action device port  slot  data a
0  1  2   3   4      5      6     7     8
*/
void parseData(){
  isStart = false;
  int idx = readBuffer(3);
  command_index = (uint8_t)idx;
  int action = readBuffer(4);
  int device = readBuffer(5);
  switch(action){
    case GET:{
        if(device != ULTRASONIC_SENSOR){
          writeHead();
          writeSerial(idx);
        }
        readSensor(device);
        writeEnd();
     }
     break;
     case RUN:{
       runModule(device);
       callOK();
     }
      break;
      case RESET:{
        //reset
        
        callOK();
      }
     break;
     case START:{
        //start
        callOK();
      }
     break;
  }
}
void callOK(){
    writeSerial(0xff);
    writeSerial(0x55);
    writeEnd();
}
void sendByte(char c){
  writeSerial(1);
  writeSerial(c);
}
void sendString(String s){
  int l = s.length();
  writeSerial(4);
  writeSerial(l);
  for(int i=0;i<l;i++){
    writeSerial(s.charAt(i));
  }
}
void sendFloat(float value){ 
     writeSerial(0x2);
     val.floatVal = value;
     writeSerial(val.byteVal[0]);
     writeSerial(val.byteVal[1]);
     writeSerial(val.byteVal[2]);
     writeSerial(val.byteVal[3]);
}
void sendShort(double value){
     writeSerial(3);
     valShort.shortVal = value;
     writeSerial(valShort.byteVal[0]);
     writeSerial(valShort.byteVal[1]);
}
void sendDouble(double value){
     writeSerial(2);
     valDouble.doubleVal = value;
     writeSerial(valDouble.byteVal[0]);
     writeSerial(valDouble.byteVal[1]);
     writeSerial(valDouble.byteVal[2]);
     writeSerial(valDouble.byteVal[3]);
}
short readShort(int idx){
  valShort.byteVal[0] = readBuffer(idx);
  valShort.byteVal[1] = readBuffer(idx+1);
  return valShort.shortVal; 
}
float readFloat(int idx){
  val.byteVal[0] = readBuffer(idx);
  val.byteVal[1] = readBuffer(idx+1);
  val.byteVal[2] = readBuffer(idx+2);
  val.byteVal[3] = readBuffer(idx+3);
  return val.floatVal;
}
long readLong(int idx){
  val.byteVal[0] = readBuffer(idx);
  val.byteVal[1] = readBuffer(idx+1);
  val.byteVal[2] = readBuffer(idx+2);
  val.byteVal[3] = readBuffer(idx+3);
  return val.longVal;
}

String readString(int len, int startIdx) {
  String str = "";

  for (int i = startIdx; i < (startIdx + len); i++) {
    str += (char) readBuffer(i);
  }
  return str;
}

void readString_to_BT(int idx){
  int l = readBuffer(idx);
  //mySensor.BT_SerialWrite(readBuffer(idx+1));
  /*
  for(int i=0;i<l;i++){
    mySensor.BT_SerialWrite(readBuffer(idx+i+1));
  }*/
  FirmwareDrone.BT_SerialWriteString(readString(l,8));
  //mySensor.BT_SerialWriteString("Shiba");
}


void runModule(int device){
  //0xff 0x55 0x6 0x0 0x1 0xa 0x9 0x0 0x0 0xa
  int port = readBuffer(6);
  int pin = port;
  switch(device){
   
   case DIGITAL:{
     pinMode(pin,OUTPUT);
     int v = readBuffer(7);
     digitalWrite(pin,v);
   }
   break;
   case PWM:{
     pinMode(pin,OUTPUT);
     int v = readBuffer(7);
     analogWrite(pin,v);
   }
   break;
   case TIMER:{
    lastTimeTime = millis()/1000.0; 
   }
   break;
   
   case DRONE_SENSOR_INITIALIZE:{
	  FirmwareDrone.Drone_motor_Flag = 1;
   }
   break;
   
   case DRONE_SENSOR_READ_SENSOR:{
     //if(FirmwareDrone.Drone_motor_Flag){
      //FirmwareDrone.Calibration();
      //drone_Sensor_flag = 4;
	  Sensor_Data[0] = FirmwareDrone.angle_ROLL();
	  Sensor_Data[1] = FirmwareDrone.angle_PITCH();
	  Sensor_Data[2] = FirmwareDrone.angle_YAW();
	  Sensor_Data[3] = FirmwareDrone.ALTITUDE_Data();
     //}
   }
   break;
   
   case DRONE_MOTOR_CONTROL:{
		int pwm = readShort(7);
	   pinMode(pin,OUTPUT);
	   FirmwareDrone.Drone_analogWrite(pin,pwm);
   }
   break;
   
   case DRONE_SENSOR_RESET_SENSOR:{
     //if(drone_begin_flag != 0){
      //FirmwareDrone.Calibration();
	  FirmwareDrone.sensorCalibrationSwitch = true;
     //}
   }
   break;
   
   case DRONE_SENSOR_SEND_BT:{
     //if(drone_begin_flag != 0){
       readString_to_BT(7);
     //}
   }
   break;
   
   case DRONE_SENSOR_BT_SETNAME:{
     //if(drone_begin_flag != 0){
	  int pp = readBuffer(7);
	  //mySensor.BT_SerialWrite(readBuffer(idx+1));
	  /*
	  for(int i=0;i<l;i++){
		mySensor.BT_SerialWrite(readBuffer(idx+i+1));
	  }*/
       FirmwareDrone.Bluetooth_Setting(readString(pp,8));
     //}
   }
   break;
   
   case DRONE_SENSOR_BT_PARSING_DATA:{
     //if(FirmwareDrone.Drone_motor_Flag){
		 //FirmwareDrone.readRemoteData();
		 drone_Command_flag = 1;
		 Control_Data[0] = FirmwareDrone.cmd[0];
		 Control_Data[1] = FirmwareDrone.cmd[1];
		 Control_Data[2] = FirmwareDrone.cmd[2];
		 Control_Data[3] = FirmwareDrone.cmd[3];
		 Control_Data[4] = FirmwareDrone.cmd[4];
	 //}
   }
   break;
  }
}

void readSensor(int device){
  /**************************************************
      ff 55 len idx action device port slot data a
      0  1  2   3   4      5      6    7    8
  ***************************************************/
  float value=0.0;
  int port,slot,pin;
  port = readBuffer(6);
  pin = port;
  switch(device){
   case  VERSION:{
     sendString(mVersion);
   }
   break;
   case  DIGITAL:{
     pinMode(pin,INPUT);
     sendFloat(digitalRead(pin));
   }
   break;
   case  ANALOG:{
     pin = analogs[pin];
     pinMode(pin,INPUT);
     sendFloat(analogRead(pin));
   }
   break;
   case TIMER:{
     sendFloat((float)currentTimeTime);
   }
   break;
   
   case DRONE_SENSOR_BT_AVAILABLE:{
       sendShort(FirmwareDrone.BT_Available());
   }
   break;
   
   case DRONE_SENSOR_READ_BT:{
	   //String s = String(mySensor.BT_SerialRead());
       //sendByte(mySensor.BT_SerialRead());
	   
	   String str = "";
	   str += (char)FirmwareDrone.BT_SerialRead();	   
	   sendString(str);
   }
   break;
   
   case DRONE_SENSOR_BT_CMD_READ:{
       int buff_add = readBuffer(6);
       if(buff_add >= 0 && buff_add < 5 ){
		   //sendByte(FirmwareDrone.readCmd(buff_add,FirmwareDrone.cmd[buff_add]));
		   //String str = "";
		   //str += (char)FirmwareDrone.readCmd(buff_add,FirmwareDrone.cmd[buff_add]);
		   //sendString(str);
		   
		   sendShort(Control_Data[buff_add]);
	   }
   }
   break;
   
   case DRONE_SENSOR_READ_SENSOR_DATA:{
       int buff_add = readBuffer(6);
		if(buff_add >= 0 && buff_add < 4 ){
		   //sendByte(FirmwareDrone.readCmd(buff_add,FirmwareDrone.cmd[buff_add]));
		   //String str = "";
		   //str += (char)FirmwareDrone.readCmd(buff_add,FirmwareDrone.cmd[buff_add]);
		   //sendString(str);
		   sendFloat(Sensor_Data[buff_add]);
	   }
   break;
  }
 }
}


void Firmware_setup(){
  digitalWrite(4,LOW);
  digitalWrite(13,LOW);
  digitalWrite(12,LOW);
  digitalWrite(8,LOW);
  delay(300);
  Serial.begin(115200);
  delay(500);
  Serial.print("Version: ");
  Serial.println(mVersion);
  FirmwareDrone.begin();
  //FirmwareDrone.MotorSetting(6,10,5,9);
}

void Firmware_loop(){
	   
	  if(drone_Command_flag) FirmwareDrone.readRemoteData();
	  FirmwareDrone.readSensorData();
	  FirmwareDrone.sensorCalibration();
	 
	  currentTimeTime = millis()/1000.0-lastTimeTime;
	 
	  readSerial();
	  
	  if(isAvailable){
		unsigned char c = serialRead&0xff;
		if(c==0x55&&isStart==false){
		 if(prevc==0xff){
		  index=1;
		  isStart = true;
		 }
		}else{
		  prevc = c;
		  if(isStart){
			if(index==2){
			 dataLen = c; 
			}else if(index>2){
			  dataLen--;
			}
			buffer[index]=c;
		  }
		}
		 index++;
		 if(index>51){
		  index=0; 
		  isStart=false;
		 }
		 if(isStart&&dataLen==0&&index>3){ 
			isStart = false;
			parseData(); 
			index=0;
		 }
	  }
	  
}
