#ifndef SENSORS_H_
#define SENSORS_H_

#include "BNC_CodingDrone_def.h"
#include "BNC_CodingDrone_structs.h"
/**********************************  constant loop time  ******************************/
#define LOOP_TIME 2800

#define PID_CONTROLLER 1

/////////Extern ///////////////

extern uint32_t currentTime;
extern uint16_t previousTime;
extern uint16_t cycleTime;
extern uint16_t cycleTime_TOTAL;
extern uint16_t rcTime;



extern volatile unsigned long timer0_overflow_count;

extern const char pidnames[];
extern const char boxnames[];
extern const uint8_t boxids[];

extern uint16_t calibratingA;
extern uint16_t calibratingB;
extern uint16_t calibratingG;
extern int16_t  magHold, headFreeModeHold;
extern uint8_t  vbatMin;
extern uint8_t  rcOptions[CHECKBOXITEMS];
extern int32_t  AltHold;
extern int32_t  AltVelocityHold;
extern int16_t  sonarAlt;
extern float  BaroPID;
//extern int16_t  errorAltitudeI;

extern int16_t  i2c_errors_count;
extern global_conf_t global_conf;

extern imu_t imu;
extern analog_t analog;
extern alt_t alt;
extern att_t att;

extern conf_t conf;
extern int16_t  annex650_overrun_count;
extern System_Flag f;
extern uint16_t intPowerTrigger1;

extern int16_t gyroZero[3];
extern int16_t angle[2];

extern int32_t baroPressure;
extern int16_t baroTemperature; // temp in 0.01 deg
extern int32_t baroPressureSum;

extern int16_t axisPID[3];

extern int16_t rcData[RC_CHANS];
extern int16_t rcSerial[8];
extern int16_t rcCommand[4];
extern int16_t lookupPitchRollRC[5];
extern uint16_t lookupThrottleRC[11];


// *************************
// MAIN LOOP VARIABLE
// *************************

extern int16_t rc;
extern int16_t Idle_Throttle;
extern int16_t Throttle_TakeOff_Landing;

// *************************************** end GPS common variables and defines ******************************************************************
extern volatile uint8_t  spekFrameFlags;
extern volatile uint32_t spekTimeLast;
extern uint8_t  spekFrameDone;

#if defined(OPENLRSv2MULTI)
extern uint8_t pot_P, pot_I; // OpenLRS onboard potentiometers for P and I trim or other usages
#endif

// **********************
// Automatic ACC Offset Calibration
// **********************
#if defined(INFLIGHT_ACC_CALIBRATION)
extern uint16_t InflightcalibratingA;
extern int16_t AccInflightCalibrationArmed;
extern uint16_t AccInflightCalibrationMeasurementDone;
extern uint16_t AccInflightCalibrationSavetoEEProm;
extern uint16_t AccInflightCalibrationActive;
#endif

#if defined(ARMEDTIMEWARNING)
extern uint32_t  ArmedTimeWarningMicroSeconds;
#endif


#if defined(THROTTLE_ANGLE_CORRECTION)
extern int16_t throttleAngleCorrection;
extern int8_t  cosZ;
#else
	
extern int8_t  cosZ;
#endif


extern uint8_t Serial_Parsing_Ready;
extern uint8_t rcData_Ready_to_Read;
extern uint8_t rcSerial_State;
extern uint8_t rcSerial_Size;
extern int16_t Alt_Throttle_Compensation;
extern uint8_t Impact_Flag;
extern uint8_t FailSafe_Flag;
extern int8_t Drone_FastReady_Flag;
extern uint8_t Need_Calibration_Flag;

extern uint16_t Calibration_Delay_Count;
extern uint8_t Reset_Att_Data_Flag;

///////////////////////////////////
void Gyro_getADC ();
uint8_t Read_AirPressure();

float fast_acos (float a);

void i2c_rep_start(uint8_t address);
void i2c_write(uint8_t data );
void i2c_stop(void);
void i2c_write(uint8_t data );
void i2c_writeReg(uint8_t add, uint8_t reg, uint8_t val);
uint8_t i2c_readReg(uint8_t add, uint8_t reg);
uint8_t i2c_readAck();
uint8_t i2c_readNak();

void i2c_read_reg_to_buf(uint8_t add, uint8_t reg, uint8_t *buf, uint8_t size);

void config_pressure(uint8_t rate,uint8_t oversampling);
void config_temperature(uint8_t rate,uint8_t oversampling);

uint8_t getEstimatedAltitude();


int32_t mul(int16_t a, int16_t b);

#define BARO_TAB_SIZE   21

#define ACC_1G 512

#define ACCZ_25deg   (int16_t)(ACC_1G * 0.90631) // 0.90631 = cos(25deg) (cos(theta) of accZ comparison)
//#define ACCZ_Failsafe_deg   (int16_t)(ACC_1G * 0.64279) // 0.64279 = cos(50deg) (cos(theta) of accZ comparison)
#define ACCZ_Failsafe_deg   (int16_t)(ACC_1G / 2) // 0.5 = cos(60deg) (cos(theta) of accZ comparison)
#define ACC_VelScale (9.80665f / 10000.0f / ACC_1G)

// GYRO SCALE: we ignore the last 2 bits and convert it for rad/s
#define GYRO_SCALE (4 / 16.4 * PI / 180.0 / 1000000.0) //16.4 LSB = 1 deg/s

#endif /* SENSORS_H_ */
