#ifndef STRUCTS_H_
#define STRUCTS_H_


enum Drone_State{
  STATE_IDLE,
  COMM_READY,
  READY_TO_TAKEOFF,
  IN_FLIGHT,
  FAIL_SAFE,
  LANDING,
  LOWBAT
};

enum rc {
  ROLL,
  PITCH,
  YAW,
  THROTTLE,
  AUX1,
  AUX2,
  AUX3,
  AUX4,
  AUX5,
  AUX6,
  AUX7,
  AUX8
};

enum pid {
  PIDROLL,
  PIDPITCH,
  PIDYAW,
  PIDALT,
  PIDPOS,
  PIDPOSR,
  PIDNAVR,
  PIDLEVEL,
  PIDMAG,
  PIDVEL,     // not used currently
  PIDITEMS
};

enum box {
  BOXARM,
  CHECKBOXITEMS
};

typedef struct {
  int16_t  accSmooth[3];
  int16_t  gyroData[3];
  int16_t  magADC[3];
  int16_t  gyroADC[3];
  int16_t  accADC[3];
  float   DCMgb[3][3];
  int16_t  gyroADC_offset[3];
  int16_t Yaw_Before;         
} imu_t;

typedef struct {
  float  vbat;               // battery voltage in 0.1V steps
  uint16_t intPowerMeterSum;
  uint16_t rssi;              // range: [0;1023]
  uint16_t amperage;          // 1unit == 100mA
  uint16_t watts;             // 1unit == 1W
} analog_t;

typedef struct {
  float  FeedbackData;             // in cm
  float  EstAlt;             // in cm
  float  vario;              // variometer in cm/s
  float  EstAlt_before;             // in cm
  float  velocity;             // in cm
  float  tempvelocity;             // in cm
  float  accLPF;             // in cm
  float  tempaltitude;             // in cm
  float  velocity_before;             // in cm
  char direction;
  float Alt_error[3];              // variometer in cm/s
  float Vel_error[3];              // variometer in cm/s
} alt_t;

typedef struct {
  int32_t  EstAlt;             // in cm
  int32_t  Hold;             // in cm
} alt_trans;

typedef struct {
  int16_t angle[2];            // absolute angle inclination in multiple of 0.1 degree    180 deg = 1800
  int16_t heading;             // variometer in cm/s
  float Angle_Feedback;             // in cm
  float Angle_Z;             // in cm
  int16_t ACCZ_Feedback;             // in cm
  float ACC_Z;             // in cm
  float ACC_Z_HPF;             // in cm
  float ACC_Z_2;             // in cm
  float ACC_Z_2_HPF;             // in cm
  float Velocity;             // in cm
  float Velocity_HPF;             // in cm
  float angle_Yaw;            // in cm
  float Yaw_Hold;            // in cm
} att_t;

typedef struct {
  uint8_t OK_TO_ARM : 1 ;
  uint8_t ARMED : 1 ;
  uint8_t ACC_CALIBRATED : 1 ;

  uint8_t SMALL_ANGLES_25 : 1 ;

  uint8_t START_CALIBRATED_A: 1;
  uint8_t START_CALIBRATED_G: 1;
  uint8_t MOTOR_ON: 1;
  uint8_t SLOW_START: 1;


  uint8_t COMM_ONLINE: 1;
  uint8_t ALTITUDE_START: 1;
  uint8_t COMM_ONLINE_BEFORE: 1;
} System_Flag;

typedef struct {
  uint8_t currentSet;
  int16_t accZero[3];
  int16_t magZero[3];
  //uint16_t flashsum;
  //uint8_t checksum;      // MUST BE ON LAST POSITION OF STRUCTURE !
} global_conf_t;

struct pid_ {
  uint8_t P8;
  uint8_t I8;
  uint8_t D8;
};


typedef struct {
  pid_    pid[PIDITEMS];
  uint8_t rcRate8;
  uint8_t rcExpo8;
  uint8_t rollPitchRate;
  uint8_t yawRate;
  uint8_t dynThrPID;
  uint8_t thrMid8;
  uint8_t thrExpo8;
  int16_t angleTrim[2];

  uint16_t activate[CHECKBOXITEMS];

  uint8_t powerTrigger1;
  int16_t failsafe_throttle;
  uint8_t vbatscale;
  uint8_t vbatlevel_warn1;
  uint8_t vbatlevel_warn2;
  uint8_t vbatlevel_crit;

#ifdef MMGYRO
  uint8_t mmgyro;
#endif
#ifdef ARMEDTIMEWARNING
  uint16_t armedtimewarning;
#endif
  int16_t minthrottle;
  //uint8_t  checksum;      // MUST BE ON LAST POSITION OF CONF STRUCTURE !
} conf_t;



#endif /* STRUCTS_H_ */
